@@ -10,7 +10,7 @@ BEGIN
 
     @ISA = 'Exporter';
 
-    $VERSION = '0.91';
+    $VERSION = '0.92';
 
     my %tags =
         ( types =>
@@ -703,6 +703,27 @@ Support questions can be sent to Dave at autarch@urth.org.
 
 The code repository is at https://svn.urth.org/svn/Params-Validate/
 
+=head1 DONATIONS
+
+If you'd like to thank me for the work I've done on this module,
+please consider making a "donation" to me via PayPal. I spend a lot of
+free time creating free software, and would appreciate any support
+you'd care to offer.
+
+Please note that B<I am not suggesting that you must do this> in order
+for me to continue working on this particular software. I will
+continue to do so, inasmuch as I have in the past, for as long as it
+interests me.
+
+Similarly, a donation made in this way will probably not make me work
+on this software much more, unless I get so many donations that I can
+consider working on free software full time, which seems unlikely at
+best.
+
+To donate, log into PayPal and send money to autarch@urth.org or use
+the button on this page:
+L<http://www.urth.org/~autarch/fs-donation.html>
+
 =head1 AUTHORS
 
 Dave Rolsky, <autarch@urth.org> and Ilya Martynov <ilya@martynov.org>
