@@ -1,12 +1,14 @@
-# Copyright (c) 2000-2004 Dave Rolsky
-# All rights reserved.
-# This program is free software; you can redistribute it and/or
-# modify it under the same terms as Perl itself.  See the LICENSE
-# file that comes with this distribution for more details.
-
 package Params::Validate;
 
 use strict;
+use warnings;
+
+use Scalar::Util ();
+
+# suppress subroutine redefined warnings if we tried to load the XS
+# version and failed.
+no warnings 'redefine';
+
 
 BEGIN
 {
@@ -210,28 +212,25 @@ sub validate (\@$)
     my $specs = $_[1];
     local $options = _get_options( (caller(0))[0] ) unless defined $options;
 
-    unless ( $NO_VALIDATION )
+    if ( ref $p eq 'ARRAY' )
     {
-        if ( ref $p eq 'ARRAY' )
+        # we were called as validate( @_, ... ) where @_ has a
+        # single element, a hash reference
+        if ( ref $p->[0] )
         {
-            # we were called as validate( @_, ... ) where @_ has a
-            # single element, a hash reference
-            if ( ref $p->[0] )
-            {
-                $p = $p->[0];
-            }
-            elsif ( @$p % 2 )
-            {
-                my $called = _get_called();
+            $p = $p->[0];
+        }
+        elsif ( @$p % 2 )
+        {
+            my $called = _get_called();
 
-                $options->{on_fail}->
-                    ( "Odd number of parameters in call to $called " .
-                      "when named parameters were expected\n" );
-            }
-            else
-            {
-                $p = {@$p};
-            }
+            $options->{on_fail}->
+                ( "Odd number of parameters in call to $called " .
+                  "when named parameters were expected\n" );
+        }
+        else
+        {
+            $p = {@$p};
         }
     }
 
@@ -289,10 +288,10 @@ sub validate (\@$)
 
     unless ( $options->{allow_extra} )
     {
-        my $called = _get_called();
-
 	if ( my @unmentioned = grep { ! exists $specs->{$_} } keys %$p )
 	{
+            my $called = _get_called();
+
 	    $options->{on_fail}->
                 ( "The following parameter" . (@unmentioned > 1 ? 's were' : ' was') .
                   " passed in the call to $called but " .
@@ -428,7 +427,7 @@ sub _normalize_callback
 sub _normalize_named
 {
     # intentional copy so we don't destroy original
-    my %h = %{ $_[0] };
+    my %h = ( ref $_[0] ) =~ /ARRAY/ ? @{ $_[0] } : %{ $_[0] };
 
     if ( $options->{ignore_case} )
     {
@@ -454,6 +453,25 @@ sub _validate_one_param
 
     if ( exists $spec->{type} )
     {
+        unless ( defined $spec->{type}
+                 && Scalar::Util::looks_like_number( $spec->{type} )
+                 && $spec->{type} > 0 )
+        {
+            my $msg = "$id has a type specification which is not a number. It is ";
+            if ( defined $spec->{type} )
+            {
+                $msg .= "a string - $spec->{type}";
+            }
+            else
+            {
+                $msg .= "undef";
+            }
+
+            $msg .= ".\n Use the constants exported by Params::Validate to declare types.";
+
+            $options->{on_fail}->($msg);
+        }
+
 	unless ( _get_type($value) & $spec->{type} )
 	{
             my $type = _get_type($value);
@@ -635,15 +653,16 @@ sub _validate_one_param
 
     sub _get_options
     {
-	my ( $caller, %override ) = @_;
+        my $caller = shift;
 
-        if ( %override )
+        if (@_)
         {
+
             return
                 ( $OPTIONS{$caller} ?
                   { %{ $OPTIONS{$caller} },
-                    %override } :
-                  { %defaults, %override }
+                    @_ } :
+                  { %defaults, @_ }
                 );
         }
         else
@@ -693,8 +712,8 @@ Params::Validate documentation for details.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2004 David Rolsky.  All rights reserved.  This program
-is free software; you can redistribute it and/or modify it under the
-same terms as Perl itself.
+Copyright (c) 2004-2007 David Rolsky.  All rights reserved.  This
+program is free software; you can redistribute it and/or modify it
+under the same terms as Perl itself.
 
 =cut
